from matplotlib import pyplot

with open('stats.txt') as f:
    h = f.readline().split()
    assert(h[0]) == 'midpoint'
    data = {k: [] for k in h[1:]}
    x = []
    for line in f:
        d = line.split()
        assert len(d) == len(h)
        d = dict(zip(h, d))
        x.append(float(d['midpoint'])/1000000)
        for k in data:
            try: data[k].append(int(d[k]))
            except ValueError: data[k].append(float(d[k]))

for i in range(len(x)):
    data['thetaW'][i] /= data['lseff'][i]

fig, axes = pyplot.subplots(3, 1)
fig.set_figheight(fig.get_figheight()*3)
fig.set_figwidth(fig.get_figwidth()*4)

axes[0].plot(x, data['thetaW'], 'ko-', mfc='w')
axes[1].plot(x, data['D'], 'ko-', mfc='w')
axes[2].plot(x, data['FstWC'], 'ko-', mfc='w')
axes[2].set_xlabel('Position on chr.15 (Mbp)').set_size(20)
axes[0].tick_params(axis='both', size=10, labelsize=15)
axes[1].tick_params(axis='both', size=10, labelsize=15)
axes[2].tick_params(axis='both', size=10, labelsize=15)

axes[0].text(2.1, max(data['thetaW']), '$\hat{\\theta}_W$', fontsize=25, va='top')
axes[1].text(2.1, max(data['D']), 'Tajima\'s $D$', fontsize=25, va='top')
axes[2].text(2.1, max(data['FstWC']), '$\hat{\\theta}$ ($F_{ST}$)', fontsize=25, va='top')

fig.savefig('plot.png', transparent=True)
